/**************************************************************************/
/*                                                                        */
/*  HP2TIF, a program for converting the output for the printer from the  */
/*          HP 48SX to the TIFF format for use in other programs          */
/*       by Chris Adams - gt8741b@prism.gatech.edu                        */
/*                                                                        */
/*   You may use this code in any way you wish, as long as I get credit   */
/*                                                                        */
/**************************************************************************/
/*                                                                        */
/* version 1.0 - Initial release, in response to my own wants and a       */
/*               message on the USENET group comp.sys.hp48                */
/*									  */
/**************************************************************************/

#include "hp2tif.h"
#include <stdio.h>

int rotate (int x, int pos);

int main (int argc, char *argv[])
{
	FILE *fin, *fout;
	hp_print hp_out[8];
	int index, ind2, ind2b, ind3;

	if (argc != 3)
	{
		fprintf (stderr, "Invalid number of arguments.\n");
		fprintf (stderr, "Usage:  %s <infile> <outfile>\n",argv[0]);
		exit (1);
	}

	fin = fopen (argv[1],"rb");
	if (!fin)
	{
		perror ("Can't open input file");
		exit (2);
	}

	fout = fopen (argv[2],"wb");
	if (!fout)
	{
		perror ("Can't open output file");
		exit (3);
	}

	do
		index = fgetc(fin);
	while (index != 27 && !feof(fin));
	ungetc (index, fin);

	fread (&hp_out, sizeof (hp_print), 8, fin);

	for (index=0; index<8; index++)
		if (hp_out[index].esc_code[0] != 27 &&
			hp_out[index].esc_code[1] != 131)
		{
			fprintf (stderr, "Input file not in correct format\n");
			exit (4);
		}

	fclose (fin);

	fwrite (&tiff_header, 234, 1, fout);

	for (ind3=0; ind3<8; ind3++)
		for (ind2=1, ind2b=7; ind2<256; ind2*=2, ind2b--)
		{
			for (index=0; index<16; index++)
				fputc (rotate((hp_out[ind3].data[index*8]&ind2),ind2b) +
					rotate((hp_out[ind3].data[index*8+1]&ind2),(ind2b-1)) +
					rotate((hp_out[ind3].data[index*8+2]&ind2),(ind2b-2)) +
					rotate((hp_out[ind3].data[index*8+3]&ind2),(ind2b-3)) +
					rotate((hp_out[ind3].data[index*8+4]&ind2),(ind2b-4)) +
					rotate((hp_out[ind3].data[index*8+5]&ind2),(ind2b-5)) +
					rotate((hp_out[ind3].data[index*8+6]&ind2),(ind2b-6)) +
					rotate((hp_out[ind3].data[index*8+7]&ind2),(ind2b-7)), fout);
			fputc (rotate((hp_out[ind3].data[128]&ind2),ind2b) +
				rotate((hp_out[ind3].data[129]&ind2),(ind2b-1)) +
				rotate((hp_out[ind3].data[130]&ind2),(ind2b-2)) +
				rotate((hp_out[ind3].data[131]&ind2),(ind2b-3)), fout);
		}

	fclose (fout);

	return 0;
}

int rotate (int x, int pos)
{
	if (pos >= 0)
		return x << pos;
	else
		return x >> (pos * -1);
}